;--
;
; Copyright (c) 2015 George M. Gallant, Jr.
;
; Permission is hereby granted, free of charge, to any person obtaining a copy
; of this software and associated documentation files (the "Software"), to deal
; in the Software without restriction, including without limitation the rights
; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
; copies of the Software, and to permit persons to whom the Software is
; furnished to do so, subject to the following conditions:
;
; The above copyright notice and this permission notice shall be included in
; all copies or substantial portions of the Software.
;
; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
; THE SOFTWARE.
;
;--
		include	"pov.inc"

		global	hpIsr, lpIsr
		
		extern	msgBuf

		udata_acs
		
lp_wtmp:	res	1
lp_stmp:	res	1

hp_stmp:	res	1
hp_wtmp:	res	1
hp_fsr0l:	res	1
hp_fsr0h:	res	1
tmr3l:		res	1
tmr3h:		res	1
		
dspByteCnt:	res	1
dspBytePtr:	res	2
	

		code
;--
;
;	High Priority ISR
;
;	This is the ISR timer service for the pixel writes. The variable delay_1
;	contains a counter which is the number of ticks prior to outputting the first
;	data byte.
;
;	Notes:  1. We could speed thing up a bit by
;		  a. Reserving FSR0 for just pixel pointer. No need to save/reload/restore
;		  b. Use the FAST option to retfie
;		  c. Don't check if TMR2 caused the interrupt
;		2. Perhaps this should be the Low Priority
;		3. Disable interrupts after the last pixel has been output
;--
hpIsr:		movff	STATUS, hp_stmp			; save the minimum needed to
		movff	WREG, hp_wtmp			; service the interrupt
		movff	FSR0L, hp_fsr0l
		movff	FSR0H, hp_fsr0h

;		btfss	PIR1, TMR2IF			; did timer 2 cause the interrupt
;		bra	@hp4

		bcf	PIR1, TMR2IF			; clear the cause

		movf	dspByteCnt, w
		bz	@hp1
		decf	dspByteCnt, f
		
		movff	dspBytePtr+0, FSR0L
		movff	dspBytePtr+1, FSR0H

		movf	POSTINC0, w
		movwf	LATC

		movff	FSR0L, dspBytePtr+0
		movff	FSR0H, dspBytePtr+1
		bra	@hp4

@hp1:		movlw	0xFF				; tun off all leds
		movwf	LATC
		bcf	T2CON, TMR2ON			; disable timer

@hp4:		movff	hp_fsr0h, FSR0H
		movff	hp_fsr0l, FSR0L
		movff	hp_wtmp, WREG
		movff	hp_stmp, STATUS
		retfie

;--
;
;	Low priority ISR
;
;--
lpIsr:		movff	STATUS, lp_stmp
		movwf	lp_wtmp

		movff	TMR3L, tmr3l			; get current rotational counter
		movff	TMR3H, tmr3h
		
		clrf	TMR3H				; reset rotational counter
		clrf	TMR3L
		
		movf	tmr3h, w			; get high byte of rotation timer
		rrncf	WREG, w
		rrncf	WREG, w
		andlw	0x3F
		movwf	PR2				; reset the period reload
		clrf	TMR2				; clear the period count
		bsf	T2CON, TMR2ON			; enable the counter
		
		
		movlw	high(msgBuf)			; reset message pointer
		movwf	dspBytePtr+1
		movlw	low(msgBuf)
		movwf	dspBytePtr+0
		
		movlw	198
		movwf	dspByteCnt			; reset message columns count
			
		bcf	INTCON3, INT2IF			; clear the cause

		movf	lp_wtmp, W
		movff	lp_stmp, STATUS
		retfie

		end
		
